package es.optsicom.problem.gop;



import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import es.optsicom.lib.Instance;
import es.optsicom.lib.Problem;
import es.optsicom.lib.instancefile.InstanceFile;
import es.optsicom.problem.gop.function.Function;
import es.optsicom.problem.gop.instances.FunctionInstanceFile;
import es.optsicom.problem.gop.util.SpecialIssueLib;

public class GOPInstance implements Instance {

	public static boolean EVALUATIONS_LIMITED = true;
	public static boolean USE_JNA_EVALUATIONS = true;
	public static double DEFAULT_STEP_SIZE = 0.01;
	private static final int DEFAULT_NUM_STEPS = 100;
	public static final boolean SAVE_EVALUATION_EVENTS = false;
	
	private Function function;
	private InstanceFile instanceFile;
	private int maxNumberEvaluations;
	private int numEvaluations = 0;

	private double stepSize = DEFAULT_STEP_SIZE;
	private int numSteps = DEFAULT_NUM_STEPS;
	private int scoredLSCalls = 0;
	
	public GOPInstance(InstanceFile instanceFile) {
		this.function = ((FunctionInstanceFile)instanceFile).getFunction();
		this.instanceFile = instanceFile;
		this.maxNumberEvaluations = 5000 * function.getDimension();
	}
	
	@Override
	public String getId() {
		return function.getName();
	}

	@Override
	public InstanceFile getInstanceFile() {
		// Add a wrap of InstanceFile that references the Function
		return instanceFile;
	}

	@Override
	public Problem getProblem() {
		return GOPProblem.getInstance();
	}

	public int getDimension() {
		return function.getDimension();
	}
	
	public double getBoundsWidth() {
		return (function.getUpperBound() - function.getLowerBound());
	}

	public void evaluate(GOPSolution solution) {
		numEvaluations++;
		
		if(USE_JNA_EVALUATIONS) {
			String cFunctionName = "Double_" + function.getcName();
			Method m;
			try {
				m = SpecialIssueLib.class.getMethod(cFunctionName, int.class, double[].class);
				Double value = (Double) m.invoke(SpecialIssueLib.INSTANCE, solution.getInstance().getDimension(), solution.getVariables());
				solution.setWeight(value);
			} catch (Exception e) {
				throw new Error(e);
			}
		} else {
			solution.setWeight(function.evaluate(solution));
		}
		
		if(EVALUATIONS_LIMITED && numEvaluations > maxNumberEvaluations) {
			throw new MaxNumberEvaluationsExceededException();
		}
	}

	public Function getFunction() {
		return function;
	}

	public int getMaxNumberEvaluations() {
		return maxNumberEvaluations;
	}
	
	public int getNumEvaluations() {
		return numEvaluations;
	}

	public void resetEvaluations() {
		this.numEvaluations = 0;
	}

	public double getStepSize() {
		return stepSize;
	}
	
	public void setStepSize(double stepSize) {
		this.stepSize = stepSize;
	}

	public int getNumSteps() {
		return numSteps;
	}
	
	public void setNumSteps(int numSteps) {
		this.numSteps = numSteps;
	}

	// This is to calculate how many times is the ScoredLS method called in average
	public void incrementScoredLS() {
		scoredLSCalls++;		
	}
	
	public void resetScoredLSCounter() {
		scoredLSCalls = 0;
	}

	public int getScoredLSCalls() {
		return scoredLSCalls;
	}
}
