package es.optsicom.problem.gop.analysis;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import es.optsicom.lib.InstanceDescription;
import es.optsicom.lib.analysis.AbstractAnalyzer;
import es.optsicom.lib.analysis.table.Cell;
import es.optsicom.lib.analysis.table.CellFormat;
import es.optsicom.lib.analysis.table.ComplexTable;
import es.optsicom.lib.analysis.table.NumberFormat;
import es.optsicom.lib.analysis.table.SheetTable;
import es.optsicom.lib.analysis.table.SimpleSheetCreator;
import es.optsicom.lib.analysis.table.TextSpaceSheetFormatter;
import es.optsicom.lib.analysis.table.TextTabSheetFormatter;
import es.optsicom.lib.analysis.table.NumberFormat.NumberType;
import es.optsicom.lib.experiment.Event;
import es.optsicom.lib.experiment.Execution;
import es.optsicom.lib.experiment.IMExecutions;
import es.optsicom.lib.experiment.MethodDescription;
import es.optsicom.lib.tablecreator.InstanceTableCreator;
import es.optsicom.lib.tablecreator.LastEventRP;
import es.optsicom.lib.tablecreator.RawProcessor;
import es.optsicom.lib.tablecreator.SummarizeColumn;
import es.optsicom.lib.tablecreator.ValueProvider;
import es.optsicom.lib.tablecreator.LastEventRP.Source;
import es.optsicom.lib.util.BestMode;
import es.optsicom.lib.util.Log;
import es.optsicom.lib.util.RelativizeMode;
import es.optsicom.lib.util.SummarizeMode;

public class GapAnalyzer extends AbstractAnalyzer {

	List<GapRP2> rawProcessors = new ArrayList<GapRP2>();
	
	public GapAnalyzer(BestMode bestMode) {
		super(bestMode);
		GapRP2 gapRPMin = new GapRP2(new ValueProvider(new File("best_values", "best_values.txt")));
		gapRPMin.setSummarizeMode(SummarizeMode.MIN);
		rawProcessors.add(gapRPMin);
		GapRP2 gapRPMax = new GapRP2(new ValueProvider(new File("best_values", "best_values.txt")));
		gapRPMax.setSummarizeMode(SummarizeMode.MAX);
		rawProcessors.add(gapRPMax);
		GapRP2 gapRPAvg = new GapRP2(new ValueProvider(new File("best_values", "best_values.txt")));
		gapRPAvg.setSummarizeMode(SummarizeMode.AVERAGE);
		rawProcessors.add(gapRPAvg);
		GapRP2 gapRPMedian = new GapRP2(new ValueProvider(new File("best_values", "best_values.txt")));
		gapRPMedian.setSummarizeMode(SummarizeMode.MEDIAN);
		rawProcessors.add(gapRPMedian);
	}
	
	public GapAnalyzer(BestMode bestMode, GapRP2...gapRP2s) {
		super(bestMode);
		rawProcessors.addAll(Arrays.asList(gapRP2s));
	}

	@Override
	protected void buildAnalysis(List<IMExecutions> imExecutionsList) {
		SheetTable table = new SheetTable(imExecutionsList.size()+2, 1 + rawProcessors.size()*imExecutionsList.get(0).getMethods().size());
		
		// Column titles
		int column = 1;
		for(MethodDescription md : imExecutionsList.get(0).getMethods()) {
			table.setCell(0, column, new Cell(md.getId()));
			for(GapRP2 gapRP : rawProcessors) {
				table.setCell(1, column, new Cell(gapRP.getSummarizeMode().toString()));
				column++;
			}
		}
		
		int row = 2;
		for(IMExecutions instanceMethodsExecs : imExecutionsList) {
			InstanceDescription instance = instanceMethodsExecs.getInstanceDescription();
			table.setCell(row, 0, new Cell(instance.getId()));
			Log.debugln(instance.getId());
			int col = 1;
			for(MethodDescription method : instanceMethodsExecs.getExecsMap().keySet()) {
				List<Execution> methodExecutions = instanceMethodsExecs.getExecsMap().get(method);
				// De estas ejecuciones queremos sacar el max, min, avg, median
				for(GapRP2 gapRP : rawProcessors) {
					double[] minValues = gapRP.cookEvents(methodExecutions);
					table.setCell(row, col, new Cell(minValues[0], new NumberFormat(NumberType.DECIMAL)));
					col++;
				}
			}
			row++;
		}
		
//		TextSpaceSheetFormatter f = new TextSpaceSheetFormatter();
//		f.format(table);
//		System.out.println(f.getFormattedTable());

		TextTabSheetFormatter f2 = new TextTabSheetFormatter();
		f2.format(table);
		System.out.println(f2.getFormattedTable());

	}
	
//	@Override
	protected void buildAnalysis2(List<IMExecutions> imExecutionsList) {
		InstanceTableCreator creator = new InstanceTableCreator();
		
		GapRP2 gapRPMin = new GapRP2(new ValueProvider(new File("best_values", "best_values.txt")));
		gapRPMin.setSummarizeMode(SummarizeMode.MIN);
		creator.addSimpleRawProcessor(gapRPMin, new SummarizeColumn(SummarizeMode.FIRST, NumberType.DECIMAL));
		
		GapRP2 gapRPMax = new GapRP2(new ValueProvider(new File("best_values", "best_values.txt")));
		gapRPMax.setSummarizeMode(SummarizeMode.MAX);
		creator.addSimpleRawProcessor(gapRPMax, new SummarizeColumn(SummarizeMode.FIRST, NumberType.DECIMAL));

		GapRP2 gapRPAvg = new GapRP2(new ValueProvider(new File("best_values", "best_values.txt")));
		gapRPAvg.setSummarizeMode(SummarizeMode.AVERAGE);
		creator.addSimpleRawProcessor(gapRPAvg, new SummarizeColumn(SummarizeMode.FIRST, NumberType.DECIMAL));

		GapRP2 gapRPMedian = new GapRP2(new ValueProvider(new File("best_values", "best_values.txt")));
		gapRPMedian.setSummarizeMode(SummarizeMode.MEDIAN);
		creator.addSimpleRawProcessor(gapRPMedian, new SummarizeColumn(SummarizeMode.FIRST, NumberType.DECIMAL));

		creator.addSimpleRawProcessor(new LastEventRP(Event.FINISH_TIME_EVENT).setSource(Source.TIMESTAMP), new SummarizeColumn(SummarizeMode.AVERAGE, NumberType.TIME));
		
		creator.setImExcesList(imExecutionsList);
		
		ComplexTable ct = creator.buildTable(RelativizeMode.EXPERIMENT);
		
		SheetTable st = new SimpleSheetCreator().createSheet(ct);

//		TextSpaceSheetFormatter f = new TextSpaceSheetFormatter();
//		f.format(st);
//		System.out.println(f.getFormattedTable());

		TextTabSheetFormatter f2 = new TextTabSheetFormatter();
		f2.format(st);
		System.out.println(f2.getFormattedTable());
	}

}
