package es.optsicom.problem.gop.analysis;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;

import es.optsicom.lib.InstanceDescription;
import es.optsicom.lib.analysis.AbstractAnalyzer;
import es.optsicom.lib.analysis.table.Cell;
import es.optsicom.lib.analysis.table.CellFormat;
import es.optsicom.lib.analysis.table.ComplexTable;
import es.optsicom.lib.analysis.table.NumberFormat;
import es.optsicom.lib.analysis.table.SheetTable;
import es.optsicom.lib.analysis.table.SimpleSheetCreator;
import es.optsicom.lib.analysis.table.TextSpaceSheetFormatter;
import es.optsicom.lib.analysis.table.TextTabSheetFormatter;
import es.optsicom.lib.analysis.table.NumberFormat.NumberType;
import es.optsicom.lib.experiment.Event;
import es.optsicom.lib.experiment.Execution;
import es.optsicom.lib.experiment.IMExecutions;
import es.optsicom.lib.experiment.MethodDescription;
import es.optsicom.lib.tablecreator.InstanceTableCreator;
import es.optsicom.lib.tablecreator.LastEventRP;
import es.optsicom.lib.tablecreator.RawProcessor;
import es.optsicom.lib.tablecreator.SummarizeColumn;
import es.optsicom.lib.tablecreator.SummarizeRawProcessor;
import es.optsicom.lib.tablecreator.ValueProvider;
import es.optsicom.lib.tablecreator.LastEventRP.Source;
import es.optsicom.lib.util.BestMode;
import es.optsicom.lib.util.Log;
import es.optsicom.lib.util.RelativizeMode;
import es.optsicom.lib.util.SummarizeMode;

/**
 * An analyzer that shows the results of a method per instance and execution. 
 * This analyzer does not performs any summarization of the values, just show the
 * last objective function value for the given method in each instance and execution.
 * 
 * @author Patxi Gortázar
 *
 */
public class InstanceExecutionAnalyzer extends AbstractAnalyzer {

	public InstanceExecutionAnalyzer(BestMode bestMode) {
		super(bestMode);
	}

	@Override
	protected void buildAnalysis(List<IMExecutions> imExecutionsList) {
		
		Set<MethodDescription> methods = imExecutionsList.get(0).getExecsMap().keySet();
		
		for(MethodDescription method : methods) {
			buildTable(imExecutionsList, method);
		}
		
	}

	private void buildTable(List<IMExecutions> imExecutionsList, MethodDescription method) {
		int numInstances = imExecutionsList.size();
		int numExecutions = imExecutionsList.get(0).getExecsMap().get(method).size();
		
		SheetTable table = new SheetTable(numInstances+2, 1 + numExecutions);
		
		// Column titles
		table.setCell(0, 0, new Cell(method));
		for(int column = 1; column <= numExecutions; column++) {
			table.setCell(1, column, new Cell(column));
		}
		
		int row = 2;
		for(IMExecutions instanceMethodsExecs : imExecutionsList) {
			InstanceDescription instance = instanceMethodsExecs.getInstanceDescription();
			table.setCell(row, 0, new Cell(instance.getId()));
			Log.debugln(instance.getId());
			int col = 1;
			for(Execution exec : instanceMethodsExecs.getExecsMap().get(method)) {
				// Get last value
				Event event = exec.getLastEvent(Event.OBJ_VALUE_EVENT);
				Double value = (Double) event.getValue();
				table.setCell(row, col, new Cell(value, new NumberFormat(NumberType.DECIMAL, 2)));
				col++;
			}
			row++;
		}
		
//		TextSpaceSheetFormatter f = new TextSpaceSheetFormatter();
//		f.format(table);
//		System.out.println(f.getFormattedTable());

		TextTabSheetFormatter f2 = new TextTabSheetFormatter();
		f2.format(table);
		System.out.println(f2.getFormattedTable());
	}
	
}
