package es.optsicom.problem.gop.approx.constructive;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import es.optsicom.lib.approx.constructive.AbstractConstructive;
import es.optsicom.lib.util.Id;
import es.optsicom.lib.util.LowestSortLimitedList;
import es.optsicom.problem.gop.GOPInstance;
import es.optsicom.problem.gop.GOPSolution;

public class TaguchiConstructive extends AbstractConstructive<GOPSolution, GOPInstance> {

	protected static int EXPERIMENTS = 81;
	protected static int FACTORS = 40;
	
	private static int DEFAULT_STEP = 20;
	
	protected static int[][] table = new int[EXPERIMENTS][FACTORS];
	protected List<GOPSolution> solutions = new ArrayList<GOPSolution>();
	private LowestSortLimitedList<GOPSolution> sortedSolutions;
	private int solutionsReturned = 0;
	protected int step;
	
	private double values[] = new double[]{0.50, 0.25, 0.75};
	
	public TaguchiConstructive() {
		this(DEFAULT_STEP);
	}
	
	public TaguchiConstructive(int step) {
		this.step = step;
	}
	
	private void loadTable() {
		try {
			BufferedReader br = new BufferedReader(new FileReader(new File("taguchi/taguchi.81.40.3.2.txt")));
			String line;
			int row = 0;
			while((line = br.readLine())!= null) {
				StringTokenizer st = new StringTokenizer(line);
				for(int i = 0; i < FACTORS; i++) {
					table[row][i] = Integer.parseInt(st.nextToken());
				}
				row++;
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}		
	}

	@Override
	public GOPSolution createSolution() {
//		return solutions.get(solutionsReturned++);
		return sortedSolutions.getList().get(solutionsReturned++);
	}
	
	public List<GOPSolution> getSolutions() {
		return solutions;
	}

	public LowestSortLimitedList<GOPSolution> getSortedSolutions() {
		return sortedSolutions;
	}
	
	@Override
	public void initSolutionCreation() {
		loadTable();

		double firstValue = instance.getFunction().getLowerBound() + values[0] * (instance.getFunction().getUpperBound() - instance.getFunction().getLowerBound());
		double secondValue = instance.getFunction().getLowerBound() + values[1] * (instance.getFunction().getUpperBound() - instance.getFunction().getLowerBound());
		double thirdValue = instance.getFunction().getLowerBound() + values[2] * (instance.getFunction().getUpperBound() - instance.getFunction().getLowerBound());
		
		double values[] = new double[]{firstValue, secondValue, thirdValue}; 
		
		int startIndex = 0;
		
		// For each subset...
		do {
			
			// For each value
			for(double value : values) {

				// For each experiment
				for(int row = 0; row < EXPERIMENTS; row++) {
					
					GOPSolution solution = new GOPSolution(instance);
					for(int var = 0; var < instance.getDimension(); var++) {
						if(var < startIndex || var >= startIndex + FACTORS) {
							solution.set(var, value);
						} else {
							solution.set(var, values[table[row][var%FACTORS]-1]);
						}
					}
					
					this.solutions.add(solution);
				}
			}
			
			startIndex += step;
			
		} while(startIndex < instance.getDimension());
		
		sortedSolutions = new LowestSortLimitedList<GOPSolution>(solutions.size());
		sortedSolutions.addAll(solutions);
	}

	@Override
	public void initSolutionCreationByNum(int numSolutions) {
		initSolutionCreation();
	}

	@Override
	public void initSolutionCreationByTime(long millis) {
	}

	@Override
	public boolean isDeterminist() {
		return true;
	}
	
	@Override
	public void setInstance(GOPInstance instance) {
		super.setInstance(instance);
		this.solutions.clear();
		this.solutionsReturned = 0;
		this.initSolutionCreation();
	}
	
	@Id
	public int getStep() {
		return step;
	}

	@Id
	public double[] getValues() {
		return values;
	}
	
	public void setValues(double[] values) {
		this.values = values;
	}

}
