package es.optsicom.problem.gop.approx.improvement;

import java.util.List;
import java.util.Map.Entry;

import es.optsicom.lib.approx.improvement.AbstractImprovementMethod;
import es.optsicom.lib.approx.improvement.ImprovementMethod;
import es.optsicom.lib.approx.improvement.ImprovingMethodListener;
import es.optsicom.lib.util.Descriptive;
import es.optsicom.lib.util.DescriptiveHelper;
import es.optsicom.lib.util.Id;
import es.optsicom.lib.util.Log;
import es.optsicom.lib.util.Properties;
import es.optsicom.problem.gop.GOPInstance;
import es.optsicom.problem.gop.GOPSolution;

public class MultipleImprovement extends AbstractImprovementMethod<GOPSolution, GOPInstance> {

	private ImprovementMethod<GOPSolution, GOPInstance>[] improvementMethods;

	public MultipleImprovement(ImprovementMethod<GOPSolution, GOPInstance>... methods) {
		this.improvementMethods = methods;
	}
	
	@Override
	public boolean internalImproveSolution(GOPSolution solution, long millis) {
//		Log.debugln("Antes de TS: " + solution.getWeight());
		for(ImprovementMethod<GOPSolution, GOPInstance> impMethod : improvementMethods) {
			boolean improved = impMethod.improveSolution(solution);
//			Log.debugln("Después: " + solution.getWeight());
			if(improved) {
				this.improveApplied = true;
			}
		}
		return improveApplied;
	}

	@Override
	public boolean isDeterminist() {
		boolean determinist = true;
		for(ImprovementMethod<GOPSolution, GOPInstance> method : improvementMethods) {
			determinist &= method.isDeterminist();
		}
		return determinist;
	}

	@Override
	public Properties getProperties() {
		Properties properties = DescriptiveHelper.createProperties(this);
		int index = 1;
		for(Descriptive descriptive : this.improvementMethods) {
			Properties impProperties = DescriptiveHelper.createProperties(descriptive);
			for(Entry<String, Object> entry : impProperties.entrySet()) {
				properties.put("implist" + index + "." + entry.getKey(), entry.getValue());
			}
			index++;
		}
		return properties;
	}
	
}
