/* ******************************************************************************
 * 
 * This file is part of Optsicom
 * 
 * License:
 *   EPL: http://www.eclipse.org/legal/epl-v10.html
 *   LGPL 3.0: http://www.gnu.org/licenses/lgpl-3.0-standalone.html
 *   See the LICENSE file in the project's top-level directory for details.
 *
 * **************************************************************************** */
package es.optsicom.problem.gop.approx.pr;

import es.optsicom.lib.util.Id;
import es.optsicom.lib.util.Log;
import es.optsicom.lib.util.RandomManager;
import es.optsicom.problem.gop.GOPInstance;
import es.optsicom.problem.gop.approx.constructive.TaguchiConstructive;
import es.optsicom.problem.gop.approx.improvement.DynamicOneVarLineSearch;
import es.optsicom.problem.gop.approx.improvement.MultipleImprovement;
import es.optsicom.problem.gop.approx.improvement.ScoredLineSearch;
import es.optsicom.problem.gop.approx.improvement.SimplexSearch;

public class GOPEvolutionaryPathRelinkingMethod extends EvolutionaryPathRelinkingMethod {
	
	private boolean dynamicStepSize = false;
	private double dynamicStepFactor = 0.01;
	private boolean changeRandomly = false;
	
	public GOPEvolutionaryPathRelinkingMethod() {
		super();
		setConstructive(new TaguchiConstructive());
		setImprovement(new MultipleImprovement(new ScoredLineSearch(new DynamicOneVarLineSearch()), new SimplexSearch()));
		setPathRelinking(new MultipleEvalStraightPathRelinking(3));
		setGlobalIter(20);
		setLocalIter(10);
		setEliteSetSize(4);
		setThresold(0.1);
		setDynamicStepSize(true);
		setChangeRandomly(false);
		setDynamicStepFactor(0.01);
	}
	
	public GOPEvolutionaryPathRelinkingMethod(int eliteSetSize) {
		this();
		setEliteSetSize(eliteSetSize);
	}
	
	public GOPEvolutionaryPathRelinkingMethod(GOPPathRelinking pr) {
		this();
		this.setPathRelinking(pr);
	}

	@Override
	protected void globalFinished() {
		if(dynamicStepSize) {
			if(changeRandomly ) {
				double value = RandomManager.getRandom().nextDouble();
				double finalStepFactor = dynamicStepFactor * value;
				this.instance.setStepSize(this.instance.getStepSize() * finalStepFactor);
			} else {
				this.instance.setStepSize(this.instance.getStepSize() * dynamicStepFactor);
			}
		}
		Log.debugln(instance.getStepSize());		
	}
	
	@Override
	public void setInstance(GOPInstance instance) {
		super.setInstance(instance);
		instance.setStepSize(0.1);
		instance.setNumSteps(20);
		instance.resetScoredLSCounter();
	}

	@Id
	public boolean isDynamicStepSize() {
		return dynamicStepSize;
	}
	
	public void setDynamicStepSize(boolean dynamicStepSize) {
		this.dynamicStepSize = dynamicStepSize;
	}

	@Id
	public double getDynamicStepFactor() {
		return dynamicStepFactor;
	}

	public void setDynamicStepFactor(double dynamicStepFactor) {
		this.dynamicStepFactor = dynamicStepFactor;
	}

	public boolean isChangeRandomly() {
		return changeRandomly;
	}

	public void setChangeRandomly(boolean changeRandomly) {
		this.changeRandomly = changeRandomly;
	}
	
	
}
