package es.optsicom.problem.gop.approx.pr;

import java.util.List;

import es.optsicom.lib.util.CollectionsUtil;
import es.optsicom.lib.util.Descriptive;
import es.optsicom.lib.util.DescriptiveHelper;
import es.optsicom.lib.util.Id;
import es.optsicom.lib.util.Log;
import es.optsicom.lib.util.Properties;
import es.optsicom.problem.gop.GOPInstance;
import es.optsicom.problem.gop.GOPSolution;

/**
 * This implementation of the Straight Linking performs multiple evaluations in each path, 
 * at points a+1/k(b-a) for k=1..kmax.
 * 
 * @author patxi
 *
 */
public class MultipleEvalStraightPathRelinking extends GOPPathRelinking implements Descriptive {

	private int kmax;

	public MultipleEvalStraightPathRelinking() {
		this(2);
	}
	
	public MultipleEvalStraightPathRelinking(int kmax) {
		super();
		this.kmax = kmax; 
	}

	@Override
	public GOPSolution createSolution(GOPSolution initiatingSolution, List<GOPSolution> guidingSolutions) {
		GOPInstance instance = initiatingSolution.getInstance();
		
		GOPSolution b = guidingSolutions.get(0);
		GOPSolution c = guidingSolutions.get(1);
		
		GOPSolution bestIntermediateSol = null;
		
		List<Integer> vars = getVarsOrdenation(instance);
		
		for(int k = 2; k <= kmax; k++) {
			GOPSolution intermediateSolution = (GOPSolution) initiatingSolution.createCopy();
			moveSolution(intermediateSolution, b, vars, k);
			
			if(bestIntermediateSol == null || intermediateSolution.isBetterThan(bestIntermediateSol)) {
				bestIntermediateSol = (GOPSolution) intermediateSolution.createCopy();
			}
			
		}
				
		GOPSolution bestGlobalSolution = bestIntermediateSol;
		for(int k = 2; k <= kmax; k++) {
			// x = bestSolution1
			GOPSolution intermediateSolution = (GOPSolution) bestIntermediateSol.createCopy();
			moveSolution(intermediateSolution, c, vars, k);
			
			if(intermediateSolution.isBetterThan(bestGlobalSolution)) {
				bestGlobalSolution = (GOPSolution) intermediateSolution.createCopy();
			}
			
		}
		
		return bestGlobalSolution;
		
	}

	private void moveSolution(GOPSolution initiatingSolution, GOPSolution guidingSolution, List<Integer> vars, int k) {
		
		for(int var = 0; var < vars.size(); var++) {
			int varIndex1 = vars.get(var);
			
			// We go x = 1/k(a+b)
//			double var1 = (initiatingSolution.get(varIndex1) + guidingSolution.get(varIndex1)) / k;
			
			// We go x = a + 1/k(b-a)
			double var1 = initiatingSolution.get(varIndex1) + (1.0/k) * (guidingSolution.get(varIndex1) - initiatingSolution.get(varIndex1));
			
			initiatingSolution.set(varIndex1, var1);
		}

	}

	@Id
	public int getKmax() {
		return kmax;
	}
	
	@Override
	public Properties getProperties() {
		return DescriptiveHelper.createProperties(this);
	}
}
