package es.optsicom.problem.gop.instances;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import es.optsicom.lib.Instance;
import es.optsicom.lib.instancefile.InstanceFile;
import es.optsicom.lib.instancefile.InstanceFileSet;
import es.optsicom.lib.instancefile.InstancesRepository;
import es.optsicom.problem.gop.GOPInstance;
import es.optsicom.problem.gop.function.Function;
import es.optsicom.problem.gop.function.cec2008.Extended_F_10;
import es.optsicom.problem.gop.function.cec2008.F12;
import es.optsicom.problem.gop.function.cec2008.F13;
import es.optsicom.problem.gop.function.cec2008.F14;
import es.optsicom.problem.gop.function.cec2008.F15;
import es.optsicom.problem.gop.function.cec2008.F18;
import es.optsicom.problem.gop.function.cec2008.F19;
import es.optsicom.problem.gop.function.cec2008.F20;
import es.optsicom.problem.gop.function.cec2008.F21;
import es.optsicom.problem.gop.function.cec2008.FBohachevsky;
import es.optsicom.problem.gop.function.cec2008.FSchaffer;
import es.optsicom.problem.gop.function.cec2008.FSchwefel1_2;
import es.optsicom.problem.gop.function.cec2008.FSchwefel2_22;
import es.optsicom.problem.gop.function.cec2008.ShiftedSchwefelProblem;
import es.optsicom.problem.gop.function.cec2008.ShiftedAckley;
import es.optsicom.problem.gop.function.cec2008.ShiftedGriewank;
import es.optsicom.problem.gop.function.cec2008.ShiftedRastrigin;
import es.optsicom.problem.gop.function.cec2008.ShiftedRosenbrock;
import es.optsicom.problem.gop.function.cec2008.ShiftedSphere;
import es.optsicom.problem.gop.function.specialissue.F16New;
import es.optsicom.problem.gop.function.specialissue.F17New;
import es.optsicom.problem.gop.function.specialissue.F18New;
import es.optsicom.problem.gop.function.specialissue.F19New;
import es.optsicom.problem.gop.function.specialissue.ShiftedExtendedF10;
import es.optsicom.problem.gop.function.specialissue.ShiftedF15;
import es.optsicom.problem.gop.function.specialissue.ShiftedFBohachevsky;
import es.optsicom.problem.gop.function.specialissue.ShiftedFSchaffer;
import es.optsicom.problem.gop.function.specialissue.ShiftedSchwefel1_2;
import es.optsicom.problem.gop.function.specialissue.ShiftedSchwefelProblem2_22;

public class GOPRepositoryInstances extends InstancesRepository {

	// private Map<String, Function> functions = new HashMap<String,
	// Function>();

	public GOPRepositoryInstances(String useCase) {
		super(useCase);
		populate(useCase);
	}

	public void populate(String useCase) {
		if ("500_1_5".equals(useCase) || "500_6_10".equals(useCase) || "500_11_15".equals(useCase) || "500_16_19".equals(useCase)) {
			this.createInstanceFileSet(useCase, "CEC2008", 500);
		} else {
			this.createInstanceFileSet(useCase, "CEC2008", 50);
			this.createInstanceFileSet(useCase, "CEC2008", 100);
			this.createInstanceFileSet(useCase, "CEC2008", 200);
			this.createInstanceFileSet(useCase, "CEC2008", 500);
			this.createInstanceFileSet(useCase, "CEC2008", 1000);
		}
	}

	public List<InstanceFile> getInstanceFileWithSizes(int... sizes) {
		List<InstanceFile> ifs = new ArrayList<InstanceFile>();
		for (int size : sizes) {
			InstanceFileSet ifSet = this.getInstanceFileSet("CEC2008/" + Integer.toString(size));
			ifs.addAll(ifSet.getInstanceFiles());
		}
		return ifs;
	}

	private void createInstanceFileSet(String useCase, String fileSetId, int dimension) {

		InstanceFileSet ifSet = new InstanceFileSet();
		ifSet.setId(fileSetId + InstancesRepository.INSTANCE_ID_PATH_SEPARATOR + dimension);

		if ("preliminary".equals(useCase)) {
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F3", new ShiftedRosenbrock(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F7", new FSchwefel2_22(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F8", new FSchwefel1_2(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F13", new F13(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F17*", new F17New(dimension)));
		} else if("paper".equals(useCase)) {
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F1", new ShiftedSphere(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F2", new ShiftedSchwefelProblem(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F3", new ShiftedRosenbrock(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F4", new ShiftedRastrigin(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F5", new ShiftedGriewank(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F6", new ShiftedAckley(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F7", new ShiftedSchwefelProblem2_22(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F8", new ShiftedSchwefel1_2(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F9", new ShiftedExtendedF10(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F10", new ShiftedFBohachevsky(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F11", new ShiftedFSchaffer(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F12", new F12(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F13", new F13(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F14", new F14(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F15", new ShiftedF15(dimension)));
			// ifSet.addInstanceFile(new FunctionInstanceFile(new
			// F16(dimension))); "F16"
			// ifSet.addInstanceFile(new FunctionInstanceFile(new
			// F17(dimension))); "F17*
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F16", new F16New(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F17*", new F17New(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F18*", new F18New(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F19*", new F19New(dimension)));
		} else if("500_1_5".equals(useCase)) {
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F1", new ShiftedSphere(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F2", new ShiftedSchwefelProblem(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F3", new ShiftedRosenbrock(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F4", new ShiftedRastrigin(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F5", new ShiftedGriewank(dimension)));
		} else if("500_6_10".equals(useCase)) {
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F6", new ShiftedAckley(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F7", new ShiftedSchwefelProblem2_22(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F8", new ShiftedSchwefel1_2(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F9", new ShiftedExtendedF10(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F10", new ShiftedFBohachevsky(dimension)));
		} else if("500_11_15".equals(useCase)) {
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F11", new ShiftedFSchaffer(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F12", new F12(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F13", new F13(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F14", new F14(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F15", new ShiftedF15(dimension)));
		} else if("500_16_19".equals(useCase)) {
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F16", new F16New(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F17*", new F17New(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F18*", new F18New(dimension)));
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F19*", new F19New(dimension)));
		} else if("F8".equals(useCase)) {
			ifSet.addInstanceFile(new FunctionInstanceFile(this, useCase, ifSet.getId(), "F8", new ShiftedSchwefel1_2(dimension)));
		}

		this.putInstanceFileSet(ifSet);

	}

	// private void createInstanceFile(int dimension, InstanceFileSet ifSet,
	// String functionName, Class<? extends Function> function) {
	// InstanceFile instanceFile = new InstanceFile(this, null, "default",
	// ifSet.getId(), functionName);
	// instanceFile.getProperties().put("dimension", dimension);
	// ifSet.addInstanceFile(instanceFile);
	// Constructor<? extends Function> cons;
	// try {
	// cons = function.getConstructor(int.class);
	// functions.put(instanceFile.getId(), cons.newInstance(dimension));
	// } catch (Exception e) {
	// e.printStackTrace();
	// }
	// }

	@Override
	public Instance loadInstance(InstanceFile instanceFile) throws IOException {
		return new GOPInstance(instanceFile);
	}

	// @Override
	// public FunctionInstanceFile loadInstanceFile(String instanceId) {
	// int indexOf = instanceId.lastIndexOf(INSTANCE_ID_PATH_SEPARATOR);
	// String instanceSetId = instanceId.substring(0, indexOf);
	//		
	// InstanceFileSet<FunctionInstanceFile, GOPInstance> instanceFile =
	// getInstanceFileSet(instanceSetId);
	// if (instanceFile == null) {
	// throw new InstancesRepositoryException("InstanceFileSet unknown for id "
	// + instanceSetId);
	// } else {
	// return instanceFile.getInstanceFile(instanceId.substring(indexOf + 1),
	// null);
	// }
	// }
}
